import { createStyleMixin, draggableMixin, componentNameValidationMixin } from '../../mixins';
import { createBaseClass } from '../../baseClasses/createBaseClass';
import { compose } from '../../helpers';
import { getComponentName } from '../../helpers/componentHelpers';
import greenVIcon from './green-v.svg';
import { TextClass } from '@descope-ui/descope-text/class';
import { ButtonClass } from '@descope-ui/descope-button/class';
import { BadgeClass } from '../descope-badge/BadgeClass';
import { IconClass } from '@descope-ui/descope-icon/class';
import { injectStyle } from '@descope-ui/common/components-helpers';

export const componentName = getComponentName('user-auth-method');
class RawUserAuthMethod extends createBaseClass({
  componentName,
  baseSelector: ':host > .root',
}) {
  constructor() {
    super();

    this.attachShadow({ mode: 'open' }).innerHTML = `
      <div class="root">
        <div class="text-wrapper">
          <slot name="method-icon"></slot>
          <descope-text st-text-align="auto" data-id="label-text" variant="body1" mode="primary"></descope-text>
        </div>

        <div class="btn-wrapper">
          <descope-button size="sm" variant="link" mode="primary">
            <slot name="button-icon"></slot>
          </descope-button>
          <div class="fulfilled-indicator">
            <vaadin-icon src=${greenVIcon}></vaadin-icon>
          </div>
          <descope-button class="hidden-btn" size="sm" variant="link" mode="primary"></descope-button>
        </div>
      </div>
		`;

    injectStyle(
      `
      :host {
					display: inline-flex;
				}

        vaadin-icon {
          color: currentcolor;
        }

        .root {
          display: flex;
          width: 100%;
          height: 100%;
          align-items: center;
        }

        .btn-wrapper {
          display: flex;
          justify-content: space-between;
          align-items: center;
          flex-grow: 0;
        }

        .text-wrapper {
          display: flex;
          align-items: center;
          flex-grow: 1;
        }

        descope-text::part(text-wrapper) {
          text-overflow: ellipsis;
          overflow: hidden;
          white-space: nowrap;
          width: initial;
        }

        descope-text {
          display: inline-flex;
        }

        .fulfilled-indicator {
          width: 1em;
          height: 1em;
          display: flex;
          align-items: center;
          padding-inline-end: 1em;
        }

        .hidden {
          display: none;
        }

        .hidden-btn {
          width: 0;
          overflow: hidden;
        }

        slot[name="method-icon"]{
          display: inline-flex;
          align-items: center;
        }
          `,
      this
    );

    this.button = this.shadowRoot.querySelector('descope-button');
    this.fulfilledIndicator = this.shadowRoot.querySelector('.fulfilled-indicator');
    this.labelText = this.shadowRoot.querySelector('descope-text[data-id="label-text"]');
  }

  onLabelChange() {
    this.labelText.innerText = this.label;
    this.labelText.setAttribute('title', this.label);
  }

  onButtonLabelChange() {
    let textSpanEle = this.button.querySelector('span');

    if (this.buttonLabel) {
      if (!textSpanEle) {
        textSpanEle = document.createElement('span');
        this.button.appendChild(textSpanEle);
      }
      textSpanEle.innerText = this.buttonLabel;
    } else if (textSpanEle) {
      this.button.removeChild(textSpanEle);
    }
  }

  onFulfilledChange() {
    this.button.classList.toggle('hidden', this.isFulfilled);
    this.fulfilledIndicator.classList.toggle('hidden', !this.isFulfilled);
  }

  get label() {
    return this.getAttribute('label') || '';
  }

  get buttonLabel() {
    return this.getAttribute('button-label') || '';
  }

  get isFulfilled() {
    return this.getAttribute('fulfilled') === 'true';
  }

  init() {
    this.onLabelChange();
    this.onButtonLabelChange();
    this.onFulfilledChange();

    this.button.addEventListener('click', () =>
      this.dispatchEvent(new CustomEvent('button-clicked', { bubbles: true, composed: true }))
    );
  }

  static get observedAttributes() {
    return ['label', 'fulfilled'].concat(super.observedAttributes);
  }

  attributeChangedCallback(name, oldValue, newValue) {
    super.attributeChangedCallback?.(name, oldValue, newValue);
    if (oldValue === newValue) {
      return;
    }

    if (name === 'label') {
      this.onLabelChange();
    } else if (name === 'fulfilled') {
      this.onFulfilledChange();
    } else if (name === 'button-label') {
      this.onButtonLabelChange();
    }
  }
}

const { host, textField, buttons, badge, textWrapper, methodIconSlot } = {
  host: { selector: () => ':host' },
  textField: { selector: 'descope-text' },
  buttons: { selector: 'descope-button' },
  badge: { selector: 'descope-badge' },
  textWrapper: { selector: ' .text-wrapper' },
  methodIconSlot: { selector: 'slot[name="method-icon"]' },
};

export const UserAuthMethodClass = compose(
  createStyleMixin({
    mappings: {
      hostWidth: { ...host, property: 'width' },
      hostMinWidth: { ...host, property: 'min-width' },
      hostDirection: [
        { ...host, property: 'direction' },
        { ...textField, property: TextClass.cssVarList.hostDirection },
        { ...buttons, property: ButtonClass.cssVarList.hostDirection },
        { ...badge, property: BadgeClass.cssVarList.hostDirection },
      ],
      labelTextWidth: { ...textField, property: 'width' },
      itemsGap: [{ property: 'gap' }, { ...textWrapper, property: 'gap' }],
      iconSize: [
        { ...methodIconSlot, property: 'width' },
        { ...methodIconSlot, property: 'height' },
      ],
      iconColor: [{ selector: () => '::slotted(*)', property: IconClass.cssVarList.fill }],
    },
  }),
  draggableMixin,
  componentNameValidationMixin
)(RawUserAuthMethod);
