import { createBaseInputClass } from '../../../../baseClasses/createBaseInputClass';
import { forwardAttrs, getComponentName } from '../../../../helpers/componentHelpers';
import CountryCodes from '../../CountryCodes';
import { comboBoxItem, parsePhoneNumber } from '../helpers';
import { AsYouType, parsePhoneNumberFromString } from 'libphonenumber-js/min';

export const componentName = getComponentName('phone-field-internal');

const commonAttrs = ['disabled', 'size', 'bordered', 'readonly'];
const countryAttrs = ['country-input-placeholder', 'default-code', 'restrict-countries'];
const phoneAttrs = ['phone-input-placeholder', 'maxlength', 'autocomplete', 'name'];
const labelTypeAttrs = ['label-type', 'country-input-label', 'label'];
const mapAttrs = {
  'country-input-label': 'label',
  'country-input-placeholder': 'placeholder',
  'phone-input-placeholder': 'placeholder',
};

const inputRelatedAttrs = [].concat(commonAttrs, countryAttrs, phoneAttrs, labelTypeAttrs);

const BaseInputClass = createBaseInputClass({ componentName, baseSelector: 'div' });

class PhoneFieldInternal extends BaseInputClass {
  static get observedAttributes() {
    return [].concat(BaseInputClass.observedAttributes || [], inputRelatedAttrs);
  }

  #ayt;

  constructor() {
    super();

    this.innerHTML = `
    <div class="wrapper">
      <descope-combo-box
        item-label-path="data-name"
        item-value-path="data-id"
      >
      ${CountryCodes.map((item) => comboBoxItem(item)).join('')}
      </descope-combo-box>
      <div class="separator"></div>
      <descope-text-field type="tel"></descope-text-field>
    </div>
    `;

    this.comboBox = this.querySelector('descope-combo-box');
    this.textField = this.querySelector('descope-text-field');

    this.inputs = [this.comboBox, this.textField];

    forwardAttrs(this, this.comboBox, { includeAttrs: ['label-type'] });
    forwardAttrs(this, this.textField, { includeAttrs: ['label-type', 'required'] });

    // override combo box setter to display dialCode value in input
    this.comboBox.customValueTransformFn = (val) => {
      const [, dialCode] = val?.split?.(' ') || [];
      return dialCode;
    };
  }

  // exposed from main component
  get countryCodeInputData() {
    return this.comboBox.items;
  }

  // exposed from main component
  get countryCodeValue() {
    return this.comboBox.shadowRoot.querySelector('input').value;
  }

  // exposed from main component
  get phoneNumberInputEle() {
    return this.textField.shadowRoot.querySelector('input');
  }

  get allowAlphanumericInput() {
    return this.getAttribute('allow-alphanumeric-input') === 'true';
  }

  get defaultCode() {
    return this.getAttribute('default-code');
  }

  get selectionStart() {
    return this.textField.selectionStart;
  }

  get minLength() {
    return parseInt(this.getAttribute('minlength'), 10) || 0;
  }

  get selectedCountryCode() {
    return this.comboBox?.selectedItem?.getAttribute('data-country-code');
  }

  get restrictCountries() {
    const attr = this.getAttribute('restrict-countries');
    return attr?.split(',').filter(Boolean) || [];
  }

  get isFormatValue() {
    return this.getAttribute('format-value') === 'true';
  }

  // `strict validation` enforces value parsing with libphonenumber-js
  get isStrictValidation() {
    return this.getAttribute('strict-validation') === 'true';
  }

  get value() {
    if (!this.comboBox.value || !this.textField.value) {
      return '';
    }

    const [countryCode, phoneNumber] = parsePhoneNumber(
      `${this.comboBox.value}-${this.textField.value}`
    );

    return `${countryCode || this.comboBox.value}-${phoneNumber || this.textField.value}`;
  }

  set value(val) {
    const [countryCode, nationalNumber] = parsePhoneNumber(val);

    this.#setCountryCode(countryCode);
    this.#setNationalNumber(nationalNumber);
  }

  init() {
    this.addEventListener('focus', (e) => {
      // we want to ignore focus events we are dispatching
      if (e.isTrusted) this.inputs[1].focus();
    });

    super.init?.();

    this.#initInputs();
  }

  getValidity() {
    const countryCode = this.comboBox.value;
    const nationalNumer = this.textField.value;

    const isEmpty = !countryCode || !nationalNumer;
    const isValidLength = nationalNumer && nationalNumer.length >= this.minLength;

    if (this.isRequired && isEmpty) {
      return { valueMissing: true };
    }

    if (this.value) {
      if (!isValidLength) {
        return { tooShort: true };
      }

      if (this.isStrictValidation && !this.#isValidParsedValue()) {
        return { patternMismatch: true };
      }
    }

    return {};
  }

  setSelectionRange(...args) {
    this.textField.setSelectionRange(...args);
  }

  attributeChangedCallback(attrName, oldValue, newValue) {
    super.attributeChangedCallback(attrName, oldValue, newValue);

    if (oldValue !== newValue) {
      if (attrName === 'default-code' && newValue) {
        this.#handleDefaultCountryCode(newValue);
      } else if (inputRelatedAttrs.includes(attrName)) {
        const attr = mapAttrs[attrName] || attrName;

        if (commonAttrs.includes(attrName)) {
          this.inputs.forEach((input) => input.setAttribute(attr, newValue));
        } else if (countryAttrs.includes(attrName)) {
          this.comboBox.setAttribute(attr, newValue);
        } else if (phoneAttrs.includes(attrName)) {
          this.textField.setAttribute(attr, newValue);
        }
      }

      if (labelTypeAttrs.includes(attrName)) {
        this.#handleLabelTypeAttrs(attrName, newValue);
      }

      if (attrName === 'restrict-countries') {
        this.#updateComboBoxItems(this.restrictCountries);
      }
    }
  }

  #initInputs() {
    // Sanitize phone input value to filter everything but digits
    this.textField.addEventListener('input', (e) => {
      if (!this.allowAlphanumericInput) {
        const telDigitsRegExp = /^\d$/;
        const sanitizedInput = e.target.value
          .split('')
          .filter((char) => telDigitsRegExp.test(char))
          .join('');
        e.target.value = sanitizedInput;
      }
    });

    this.handleFocusEventsDispatching(this.inputs);
    this.handleInputEventDispatching();

    // verify country code item against phone number pattern and replace if needed and country is allowed
    // (e.g. +1 can be US or CA, depending on the pattern)
    this.addEventListener('input', this.#handleSameCountryCodes.bind(this));
  }

  #setCountryCode(val) {
    if (!val || val === this.selectedCountryCode) return;

    let countryCodeItem = undefined;

    if (this.value) {
      // try to parse the phone number, and set country code item according to actual dial code (e.g. `+1` can be `US` or `CA`)
      const code = this.#getCountryCodeByPhoneNumber(`${val}-${this.textField.value}`);
      countryCodeItem = this.#getCountryByCodeId(code);
    }

    // in case country code item does not exist (for example: Parsed code is CA for +1 - but Canada is not allowed)
    // then use the first option with same dial code (e.g. in that case - `US` for +1)
    if (!countryCodeItem) {
      countryCodeItem = this.#getCountryByDialCode(val);
    }

    // set country code item; in it doesn't exist in list - set `undefined`
    this.comboBox.selectedItem = countryCodeItem;
  }

  #setNationalNumber(val) {
    if (this.isFormatValue) {
      val = this.#formatNationalNumber(val);
    }

    if (this.textField.value !== val) {
      this.textField.value = val;
    }
  }

  #formatNationalNumber(nationalNumber = '') {
    // re-initialize AsYouType if country code is outdated
    if (!this.#ayt || this.#ayt.country !== this.selectedCountryCode) {
      this.#ayt = new AsYouType(this.selectedCountryCode);
    }

    // reset previous AsYouType input
    this.#ayt.reset();

    const formattedVal = this.#ayt.input(nationalNumber);

    return formattedVal || nationalNumber;
  }

  #isValidParsedValue() {
    const parsed = parsePhoneNumberFromString(this.value);

    return (
      !!parsed && // Parsed successfully (not undefined)
      !!parsed.isValid?.() && // Parsed object is valid
      !!parsed.country && // Parsed object with a country code
      !!this.#isAllowedCountry(parsed.country) // Parsed with allowed country code
    );
  }

  #isAllowedCountry(countryCode) {
    if (!this.restrictCountries.length) {
      return true;
    }

    return this.restrictCountries.includes(countryCode);
  }

  // return country item by dial code `data-id` (e.g. `+1`)
  #getCountryByDialCode(dialCode) {
    return this.comboBox.items?.find((c) => c.getAttribute('data-id') === dialCode) || undefined;
  }

  // return country item by country code `data-country-code` (e.g. `US`)
  #getCountryByCodeId(countryCode) {
    return this.comboBox.items?.find((c) => c.getAttribute('data-country-code') === countryCode);
  }

  #getCountryCodeByPhoneNumber(val) {
    if (!val) return undefined;
    const parsed = parsePhoneNumberFromString(val);
    if (!parsed?.country) return undefined;
    const foundCountryItem = this.#getCountryByCodeId(parsed.country);
    return foundCountryItem?.getAttribute('data-country-code');
  }

  #updateComboBoxItems(restrictCountries) {
    const items = restrictCountries.length
      ? CountryCodes.filter((c) => restrictCountries.includes(c.code))
      : CountryCodes;

    this.querySelector('descope-combo-box').innerHTML = items
      .map((item) => comboBoxItem(item))
      .join('');
  }

  #handleDefaultCountryCode(countryCode) {
    if (!this.comboBox.value) {
      const countryCodeItem = this.#getCountryByCodeId(countryCode);
      // When replacing the input component (inserting internal component into text-field) -
      // Vaadin resets the input's value. We use setTimeout in order to make sure this happens
      // after the reset.
      if (countryCodeItem) {
        setTimeout(() => {
          this.comboBox.selectedItem = countryCodeItem;
        });
      }
    }
  }

  #handleSameCountryCodes() {
    if (!this.value) return;

    const country = this.#getCountryCodeByPhoneNumber(this.value);

    if (!country) return;

    // re-set country code if needed (same coutnry code for different countries, e.g. +1 for US or CA)
    if (this.selectedCountryCode !== country) {
      const foundCountryItem = this.#getCountryByCodeId(country);
      // if found country is defined in country list then set it, otherwise - clear phone number
      if (foundCountryItem) {
        this.comboBox.selectedItem = foundCountryItem;
      }
    }
  }

  #handleLabelTypeAttrs(attrName, newValue) {
    // set or remove label attributes from inner text/combo components on `label-type` change
    const attr = mapAttrs[attrName] || attrName;

    if (attrName === 'label-type') {
      this.#handleLabelTypeChange(newValue);
    }
    // on inner components label attr change - set label attributes for text/combo components
    // only if label-type is `floating`
    else if (this.getAttribute('label-type') === 'floating') {
      if (attrName === 'country-input-label') {
        this.comboBox.setAttribute(attr, newValue);
      } else if (attrName === 'label') {
        this.textField.setAttribute(attr, newValue);
      }
    }
  }

  #handleLabelTypeChange(newValue) {
    if (newValue === 'floating') {
      // on change to `floating` label - set inner components `label` and `placeholder`
      this.comboBox.setAttribute('label', this.getAttribute('country-input-label') || '');
      this.comboBox.setAttribute(
        'placeholder',
        this.getAttribute('country-input-placeholder') || ''
      );
      this.textField.setAttribute('label', this.getAttribute('label') || '');
      this.textField.setAttribute(
        'placeholder',
        this.getAttribute('phone-input-placeholder') || ''
      );
    } else {
      // for other cases - reset inner components label-type and labels
      this.inputs.forEach((input) => input.removeAttribute('label'));
    }
  }
}

export default PhoneFieldInternal;
